@@ -10,13 +10,78 @@
 #include "eigenpy/fwd.hpp"
 
 #include <patchlevel.h> // For PY_MAJOR_VERSION
-#include <numpy/arrayobject.h>
 #include <iostream>
 
 #include "eigenpy/eigenpy.hpp"
 #include "eigenpy/registration.hpp"
 #include "eigenpy/map.hpp"
 
+namespace boost { namespace python { namespace converter {
+
+template<class MatType>
+struct implicit<Eigen::MatrixBase<MatType>,MatType>
+{
+  typedef Eigen::MatrixBase<MatType> Source;
+  typedef MatType Target;
+  
+  static void* convertible(PyObject* obj)
+  {
+    // Find a converter which can produce a Source instance from
+    // obj. The user has told us that Source can be converted to
+    // Target, and instantiating construct() below, ensures that
+    // at compile-time.
+    return implicit_rvalue_convertible_from_python(obj, registered<Source>::converters)
+    ? obj : 0;
+  }
+  
+  static void construct(PyObject* obj, rvalue_from_python_stage1_data* data)
+  {
+    void* storage = ((rvalue_from_python_storage<Target>*)data)->storage.bytes;
+    
+    arg_from_python<Source> get_source(obj);
+    bool convertible = get_source.convertible();
+    BOOST_VERIFY(convertible);
+    
+    new (storage) Target(get_source().derived());
+    
+    // record successful construction
+    data->convertible = storage;
+  }
+};
+
+template<class MatType>
+struct implicit<MatType,Eigen::MatrixBase<MatType> >
+{
+  typedef MatType Source;
+  typedef Eigen::MatrixBase<MatType> Target;
+  
+  static void* convertible(PyObject* obj)
+  {
+    // Find a converter which can produce a Source instance from
+    // obj. The user has told us that Source can be converted to
+    // Target, and instantiating construct() below, ensures that
+    // at compile-time.
+    return implicit_rvalue_convertible_from_python(obj, registered<Source>::converters)
+    ? obj : 0;
+  }
+  
+  static void construct(PyObject* obj, rvalue_from_python_stage1_data* data)
+  {
+    void* storage = ((rvalue_from_python_storage<Source>*)data)->storage.bytes;
+    
+    arg_from_python<Source> get_source(obj);
+    bool convertible = get_source.convertible();
+    BOOST_VERIFY(convertible);
+    
+    new (storage) Source(get_source());
+    
+    // record successful construction
+    data->convertible = storage;
+  }
+};
+
+}}} // namespace boost::python::converter
+
 #define GET_PY_ARRAY_TYPE(array) PyArray_ObjectType(reinterpret_cast<PyObject *>(array), 0)
 
 namespace eigenpy
@@ -516,36 +581,8 @@ namespace eigenpy
     }
   };
   
-  template<typename MatType>
-  struct EigenFromPy< Eigen::MatrixBase<MatType> >
-  {
-    typedef EigenFromPy<MatType> EigenFromPyDerived;
-    typedef Eigen::MatrixBase<MatType> Base;
-    
-    /// \brief Determine if pyObj can be converted into a MatType object
-    static void* convertible(PyArrayObject* pyObj)
-    {
-      return EigenFromPyDerived::convertible(pyObj);
-    }
-    
-    /// \brief Allocate memory and copy pyObj in the new storage
-    static void construct(PyObject* pyObj,
-                          bp::converter::rvalue_from_python_stage1_data* memory)
-    {
-      EigenFromPyDerived::construct(pyObj,memory);
-    }
-    
-    static void registration()
-    {
-      bp::converter::registry::push_back
-      (reinterpret_cast<void *(*)(_object *)>(&EigenFromPy::convertible),
-       &EigenFromPy::construct,bp::type_id<Base>());
-    }
-  };
-  
-#define numpy_import_array() {if (_import_array() < 0) {PyErr_Print(); PyErr_SetString(PyExc_ImportError, "numpy.core.multiarray failed to import"); } }
-  
   template<typename MatType,typename EigenEquivalentType>
+  EIGENPY_DEPRECATED
   void enableEigenPySpecific()
   {
     enableEigenPySpecific<MatType>();
@@ -560,12 +597,13 @@ namespace eigenpy
 
       // Add also conversion to Eigen::MatrixBase<MatType>
       typedef Eigen::MatrixBase<MatType> MatTypeBase;
-      EigenFromPy<MatTypeBase>::registration();
+//      bp::implicitly_convertible<MatTypeBase,MatType>();
+      bp::implicitly_convertible<MatType,MatTypeBase>();
     }
   };
 
 #if EIGEN_VERSION_AT_LEAST(3,2,0)
-  /// Template specialization for Eigen::Ref
+  // Template specialization for Eigen::Ref
   template<typename MatType>
   struct EigenFromPyConverter< eigenpy::Ref<MatType> >
   {
@@ -577,6 +615,8 @@ namespace eigenpy
     }
   };
 #endif
+
+#define numpy_import_array() {if (_import_array() < 0) {PyErr_Print(); PyErr_SetString(PyExc_ImportError, "numpy.core.multiarray failed to import"); } }
   
   template<typename MatType>
   void enableEigenPySpecific()
@@ -586,7 +626,6 @@ namespace eigenpy
     
     bp::to_python_converter<MatType,EigenToPy<MatType> >();
     EigenFromPyConverter<MatType>::registration();
-   
   }
 
 } // namespace eigenpy
